ARCHCFLAGS	=	-DSYSVSIGNAL -DNOUNIXDOM
ARCHDLIB	=
ARCHDOBJ	=
ARCHLIB		=
HASRANLIB	=	t
PVM_ARCH	=	SP2MPI
AR_FRONT    =   ar
AR_NODE     =   ar
RWBSPECIAL = |sed '/^.include \"pvm3.h\"/d' |sed '/^.include \"pvm_consts.h\"/d'
CC			=	cc
