/*
	test groups
*/

int
main(argc, argv)
int argc;
char *argv[];
{
	int mytid, mygid1, mygid2, mygid3;

	mytid = pvm_mytid();

	mygid1 = pvm_joingroup("prognosis");
	mygid2 = pvm_joingroup("negative");

	printf("joined prognosis %d, negative %d\n", mygid1, mygid2);

	mygid1 = pvm_lvgroup("prognosis");
	mygid2 = pvm_lvgroup("negative");

	printf("left prognosis %d, negative %d\n", mygid1, mygid2);
	pvm_exit();

}


