ARCHCFLAGS	=	-DSYSVSIGNAL -DRSHCOMMAND=\"/usr/bsd/rsh\" -DNOUNIXDOM -64 -TENV:large_GOT=ON -woff 1174 -DHASSTDLIB
#ARCHDLIB	=	-lsun -lmutex
ARCHDOBJ	=
#ARCHLIB	=	-lsun -lmutex
HASRANLIB	=	f
PVM_ARCH	=	SGIMP64
