ARCHCFLAGS	=	-DSYSVSIGNAL -DNOWAIT3 -DNOGETDTBLSIZ -DRSHCOMMAND=\"/usr/bin/remsh\" -DFDSETISINT
ARCHDLIB	=
ARCHDOBJ	=
ARCHLIB	=
HASRANLIB	=	f
PVM_ARCH	=	HPPAMP
RWBSPECIAL = |sed '/^.include \"pvm3.h\"/d' |sed '/^.include \"pvm_consts.h\"/d'
FC	=	fort77
