ARCHCFLAGS    =       -eft -DSYSVSTR -DNOUNIXDOM
ARCHDLIB      =       -lnsl -lsocket -lbsd -lrpc
ARCHLIB               =       -lnsl -lsocket -lbsd -lrpc
ARCHDOBJS     =
HASRANLIB     =       f
PVM_ARCH      =       UTS2
