# Intel iPSC/860 with SGI frontend
#
ARCHCFLAGS	=	-DSYSVSIGNAL -DNOWAIT3 -DRSHCOMMAND=\"/usr/bsd/rsh\" -DNOUNIXDOM
ARCHDLIB	=	-lsun -lhost
ARCHDOBJ	=
ARCHLIB		=	-lsun
HASRANLIB	=	f
PVM_ARCH	=	I860
AR_FRONT	=	ar
AR_NODE		=	ar860
CC			=	cc
