# Intel iPSC/860 with 386(SRM) frontend
# 
ARCHCFLAGS	=	-DSYSVSIGNAL -DNOWAIT3 -DNOWAITPID -DSYSVSTR -DRSHCOMMAND=\"/usr/ucb/rcmd\" -DNOUNIXDOM
ARCHDLIB	=	-lrpc -lsocket -host
ARCHDOBJ	=
ARCHLIB		=	-lrpc -lsocket
HASRANLIB	=	f
PVM_ARCH	=	I860
AR_FRONT	=	ar
AR_NODE		=	ar860
CC			=	cc
#
# Architecture type of frontend
FRONT_ARCH	=	SRM
