#include <sys/types.h>
#include <fcntl.h>

#define MAX_ARCH_NAME	31
#define MAX_EVENT_NAME	31
#define MAX_HOST_NAME	256
#define MAX_PROC_NAME	256
#define MAX_GRP_NAME	31
#define MAX_HOSTS       256

#define STRING    0
#define BYTE1     1
#define INTEGER2  2
#define INTEGER4  3
#define REAL4     4
#define COMPLEX8  5
#define REAL8     6
#define COMPLEX16 7
