/*
	test bcast node 
*/

#include <stdio.h>
#include "pvm3.h"

int
main(argc, argv)
int argc;
char *argv[];
{
	int mytid;
	char g[32];
	int cc, ngroup;
	int gid;

	mytid = pvm_mytid();
	pvm_recv(pvm_parent(), 5);
	pvm_upkstr(g);
	pvm_upkint(&ngroup, 1, 1);
	gid = pvm_joingroup(g);
	pvm_barrier(g, ngroup);
	pvm_recv(pvm_parent(), 77);
	pvm_lvgroup(g);
	pvm_exit();
}
