/*
	join and leave a lot 

*/

#include <stdio.h>
#include "pvm3.h"
#define MAXJOINS 10000
int
main(argc, argv)
int argc;
char *argv[];
{
	int mytid, mygid, njoins;
	int cnt = 0;

	mytid = pvm_mytid();
	fprintf(stderr, "%s 0x%x enrolled\n", argv[0], mytid);
	if (mytid < 0) {
		pvm_perror(argv[0]);
		return -1;
		}
        if (argc > 1)
           njoins = atoi(argv[1]);
        else
           njoins = MAXJOINS;

	/* join a group */
	while (cnt++ <= njoins) {
		if((mygid = pvm_joingroup("JoinLeave")) < 0) {
			pvm_perror("JoinLeave");
			break;
			}
		if((mygid = pvm_lvgroup("JoinLeave")) < 0) {
			pvm_perror("JoinLeave");
			break;
			}
		if ((cnt%100)==0) printf("Joined %d times\n", cnt);
		}

}
